{ **************************************************************************** }
{ *                                                                          * }
{ * TOWERING RESCUE (Gakken, LCD Card Game) Simulator                        * }
{ * by Luca "MADrigal" Antignano, July 2000 (revision: July 2017)            * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;        
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;       im_background_screen_top: TImage;

    btn_game_a_down:          TImage;       btn_game_a_top:           TImage;
    btn_game_b_down:          TImage;       btn_game_b_top:           TImage;
    btn_time_down:            TImage;       btn_time_top:             TImage;
                                            btn_timeset_top:          TImage;
                                            btn_alarm_top:            TImage;
    btn_acl_down:             TImage;       btn_acl_top:              TImage;

    btn_left_down:            TImage;       btn_left_top:             TImage;
    btn_right_down:           TImage;       btn_right_top:            TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (alarm, time/score, game a/b, etc.)
    im_panel_time_pm: TImage; im_panel_time_colon: TImage;

    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage;

    im_panel_alarm: TImage;

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    im_panel_miss_1: TImage; im_panel_miss_2: TImage;

    // on-screen items
    im_heli_0_miss_left: TImage; im_heli_1: TImage;             im_heli_2: TImage;             im_heli_3: TImage;             im_heli_4: TImage;             im_heli_5: TImage;             im_heli_6_miss_right: TImage;

    {                          } im_heli_1_rope_1: TImage;      im_heli_2_rope_1: TImage;      im_heli_3_rope_1: TImage;      im_heli_4_rope_1: TImage;      im_heli_5_rope_1: TImage;      {                           }
    {                          } im_heli_1_man_1_left: TImage;  im_heli_2_man_1_left: TImage;  im_heli_3_man_1_left: TImage;  im_heli_4_man_1_left: TImage;  im_heli_5_man_1_left: TImage;  {                           }
    {                          } im_heli_1_man_1_right: TImage; im_heli_2_man_1_right: TImage; im_heli_3_man_1_right: TImage; im_heli_4_man_1_right: TImage; im_heli_5_man_1_right: TImage; {                           }
    {                          } im_heli_1_rope_2: TImage;      {                            } im_heli_3_rope_2: TImage;      im_heli_4_rope_2: TImage;      {                            } {                           }
    {                          } im_heli_1_man_2_right: TImage; {                            } im_heli_3_man_2_right: TImage; im_heli_4_man_2_right: TImage; {                            } {                           }
    {                          } {                            } {                            } im_heli_3_rope_3: TImage;      {                            } {                            } {                           }
    {                          } {                            } {                            } im_heli_3_man_3_left: TImage;  {                            } {                            } {                           }

    {                          } im_man_1: TImage;              im_man_2: TImage;              im_man_3: TImage;              im_man_4: TImage;              im_man_5_safe_left: TImage;    {                           }
    {                          } {                            } {                            } {                            } {                            } im_man_5_safe_right: TImage;   {                           }

    {                          } im_building_1_fire: TImage;    im_building_2: TImage;         im_building_3: TImage;         im_building_4: TImage;         {                            } {                           }
    {                          } im_building_1_smoke: TImage;   im_building_2_quake: TImage;   im_building_3_quake: TImage;   im_building_4_quake: TImage;   {                            } {                           }

{ * TIMERS                                                                   * }
    Timer_ACL:              TTimer;
    Timer_Clock_1:          TTimer;
    Timer_Clock_2:          TTimer;
    Timer_Game:             TTimer;
    Timer_Game_Start:       TTimer;
    Timer_Miss:             TTimer;
    Timer_Rescue_Dropdown:  TTimer;
    Timer_Rescue_Pickup:    TTimer;

{ * DEBUG                                                                    * }
    la_debug:               TLabel;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_All_Buildings        (b: boolean);
    procedure Vid_Show_All_Heli             (b: boolean);
    procedure Vid_Show_All_Man              (b: boolean);
    procedure Vid_Show_All_Man_Safe         (b: boolean);

    procedure Vid_Show_Panel_Score          (var p:integer);
    procedure Vid_Show_Panel_Miss           (var p: integer);

    procedure Vid_Show_Building             (var p: integer; q: integer);
    procedure Vid_Show_Heli                 (b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove        (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_left_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_left_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseUp          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_game_a_down;              procedure Inp_game_a_up;
    procedure Inp_game_b_down;              procedure Inp_game_b_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_left_down;                procedure Inp_left_up;
    procedure Inp_right_down;               procedure Inp_right_up;

{ * PROCEDURES: GAME ROUTINES                                                * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Check_Collisions;
    procedure Gam_Game_Over;
    procedure Gam_Game_Start;
    procedure Gam_Set_Difficulty;
    procedure Gam_Set_Mode;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_ACLTimer                (Sender: TObject);
    procedure Timer_Clock_1Timer            (Sender: TObject);
    procedure Timer_Clock_2Timer            (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_StartTimer         (Sender: TObject);
    procedure Timer_MissTimer               (Sender: TObject);
    procedure Timer_Rescue_DropdownTimer    (Sender: TObject);
    procedure Timer_Rescue_PickupTimer      (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
{}//procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
  APP_NAME     = 'Towering Rescue (Gakken, LCD Card Game) Simulator';           // application's name (appears on taskbar and Splash screen)
{}APP_VERSION  = 'Version 5.00 * Final *';                                      // current version (appears on Splash screen)
{}APP_DATE     = 'Released by Luca "MADrigal" Antignano on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL    = 'mailto:lucantignano@gmail.com';                               // link to the email address
  URL_PAYPAL   = 'https://www.paypal.me/LucaAntignano';                         // link to the Paypal donation page
  URL_WEBSITE  = 'http://www.madrigaldesign.it/sim/';                           // link to the website

  REG_KEY      = 'Software\MADrigal\Towering50';                                // registry key

  HINT_CLICK   = 'click or press';
  HINT_GAME_A  = 'to start game A (hold to view highscore)';
  HINT_GAME_B  = 'to start game B (hold to view highscore)';
  HINT_TIME    = 'to view current time (hold to view seconds)';
  HINT_ACL     = 'to reset the game';
  HINT_ALARM   = 'button disabled';
  HINT_TIMESET = 'button disabled';

{}MENU_ITEMS = 13;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_bar,
  pfs_danger,
  pfs_miss,
  pfs_start:               PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..8]       of TImage;                         // empty sprites to host score/time/messages digits
  aIm_panel_number:        array[0..9]       of TImage;                         // digits for score/time
  aIm_panel_miss:          array[1..2]       of TImage;                         // misses counter

  aIm_heli:                array[0..6]       of TImage;
  aIm_man:                 array[1..5]       of TImage;
  aIm_man_safe:            array[1..2]       of TImage;                         // 1=left; 2=right
  aIm_buildings:           array[1..4, 1..2] of TImage;                         // x=column; y=type (building/quake/smoke/fire)

  aIm_heli_parts:          array[1..5, 1..3] of record                          // x=column; y=row
                                                  rope,
                                                  man_left,
                                                  man_right: TImage;
                                                end;

{ * arrays of integers, booleans and records * }
  aBuildings_height:       array[1..5]       of integer;                        // x=column; value=height

{ * arrays of mixed types * }
  aKeys:                   array[1..6] of record                                // keys management (values)
                                            int:  integer;
                                            word: word;
                                           end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=ACL/1; 1=ACL/2; 2=Time/1; 3=Time/2; 4=Game A; 5=Game B; 6=Game over
  iScore,                                                                       // 0-9990
  iHiScoreA,                                                                    // 0-9990
  iHiScoreB,                                                                    // 0-9990
  iMiss,                                                                        // 0-2
  iPosHeli,                                                                     // 0-6
  iMenOnBoard                                                                   // 0-2
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bControls,
  bTimePressed,
  bGameAPressed,
  bGameBPressed,
  bACLPressed,
  bKeyPressed                                                                   // left and right arrows only
                         : boolean;

  // Game variables: loops
  iLoopGameMen:            array[1..4]       of integer;

  iLoopGame,
  iLoopGameGenerate,                                                            // 10-42 (ticks between men generation)
  iLoopGameStart,
  iLoopMiss,
  iLoopRescueDropdown,
  iLoopRescuePickup
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4;

  aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2;

  // numbers
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  aIm_heli[0] := im_heli_0_miss_left; aIm_heli[1] := im_heli_1;                                aIm_heli[2] := im_heli_2;                                aIm_heli[3] := im_heli_3;                                aIm_heli[4] := im_heli_4;                                aIm_heli[5] := im_heli_5;                                aIm_heli[6] := im_heli_6_miss_right;

  {                                 } aIm_heli_parts[1, 1].rope := im_heli_1_rope_1;           aIm_heli_parts[2, 1].rope := im_heli_2_rope_1;           aIm_heli_parts[3, 1].rope := im_heli_3_rope_1;           aIm_heli_parts[4, 1].rope := im_heli_4_rope_1;           aIm_heli_parts[5, 1].rope := im_heli_5_rope_1;           {                                  }
  {                                 } aIm_heli_parts[1, 1].man_left := im_heli_1_man_1_left;   aIm_heli_parts[2, 1].man_left := im_heli_2_man_1_left;   aIm_heli_parts[3, 1].man_left := im_heli_3_man_1_left;   aIm_heli_parts[4, 1].man_left := im_heli_4_man_1_left;   aIm_heli_parts[5, 1].man_left := im_heli_5_man_1_left;   {                                  }
  {                                 } aIm_heli_parts[1, 1].man_right := im_heli_1_man_1_right; aIm_heli_parts[2, 1].man_right := im_heli_2_man_1_right; aIm_heli_parts[3, 1].man_right := im_heli_3_man_1_right; aIm_heli_parts[4, 1].man_right := im_heli_4_man_1_right; aIm_heli_parts[5, 1].man_right := im_heli_5_man_1_right; {                                  }
  {                                 } aIm_heli_parts[1, 2].rope := im_heli_1_rope_2;           {                                                      } aIm_heli_parts[3, 2].rope := im_heli_3_rope_2;           aIm_heli_parts[4, 2].rope := im_heli_4_rope_2;           {                                                      } {                                  }
  {                                 } aIm_heli_parts[1, 2].man_right := im_heli_1_man_2_right; {                                                      } aIm_heli_parts[3, 2].man_right := im_heli_3_man_2_right; aIm_heli_parts[4, 2].man_right := im_heli_4_man_2_right; {                                                      } {                                  }
  {                                 } {                                                      } {                                                      } aIm_heli_parts[3, 3].rope := im_heli_3_rope_3;           {                                                      } {                                                      } {                                  }
  {                                 } {                                                      } {                                                      } aIm_heli_parts[3, 3].man_left := im_heli_3_man_3_left;   {                                                      } {                                                      } {                                  }

  {                                 } aIm_man[1] := im_man_1;                                  aIm_man[2] := im_man_2;                                  aIm_man[3] := im_man_3;                                  aIm_man[4] := im_man_4;                                  {                                                      } {                                  }

  {                                 } aIm_buildings[1, 1] := im_building_1_smoke;              aIm_buildings[2, 1] := im_building_2;                    aIm_buildings[3, 1] := im_building_3;                    aIm_buildings[4, 1] := im_building_4;                    {                                                      } {                                  }
  {                                 } aIm_buildings[1, 2] := im_building_1_fire;               aIm_buildings[2, 2] := im_building_2_quake;              aIm_buildings[3, 2] := im_building_3_quake;              aIm_buildings[4, 2] := im_building_4_quake;              {                                                      } {                                  }

  aIm_man_safe[1] := im_man_5_safe_left; aIm_man_safe[2] := im_man_5_safe_right;

  // integer values settings
  aBuildings_height[1] := 2; aBuildings_height[2] := 1; aBuildings_height[3] := 3; aBuildings_height[4] := 2; aBuildings_height[5] := 1;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 6);                              // "ACL"    (default: "5")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 39);                             // "Left"   (default: "LEFT")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 40);                             // "Right"  (default: "RIGHT")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 6 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_game_a_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_acl_top.Hint     := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;    // "ACL"
  btn_alarm_top.Hint   := HINT_ALARM;                                                     // "Alarm"
  btn_timeset_top.Hint := HINT_TIMESET;                                                   // "Time Set"
  btn_left_top.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Left"
  btn_right_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                     // "Right"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bControls := false;                                                           // prevents hero movement
  bTimePressed := false;
  bGameAPressed := false;
  bGameBPressed := false;
  bACLPressed := false;
  bKeyPressed := false;

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_bar);  FSOUND_Sample_Free(pfs_danger);
  FSOUND_Sample_Free(pfs_miss); FSOUND_Sample_Free(pfs_start);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var i: integer;
begin
  Vid_Show_All_Buildings(b);
  Vid_Show_All_Heli(b);
  Vid_Show_All_Man(b);
  Vid_Show_All_Man_Safe(b);

  im_panel_time_pm.Visible := b;
  im_panel_time_colon.Visible := b;
  im_panel_alarm.Visible := b;

  i := Ord(b) * 2; Vid_Show_Panel_Miss(i);

  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      i := 8888; Vid_Show_Panel_Score(i);
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;
end;





procedure TForm1.Vid_Show_All_Buildings(b: boolean);
{ displays/hides all the buildings, fires, smoke                               }
  var
    i, j: integer;
begin
  for i := 1 to 4 do
    for j := 1 to 2 do
      aIm_buildings[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Heli(b: boolean);
{ displays/hides all the helicopters, ropes and hanging men                    }
  var
    i, j: integer;
begin
  for i := 0 to 6 do
    aIm_heli[i].Visible := b;

  for i := 1 to 5 do
    begin
      for j := 1 to aBuildings_height[i] do
        aIm_heli_parts[i, j].rope.Visible := b;

      aIm_heli_parts[i, 1].man_left.Visible := b;
      aIm_heli_parts[i, 1].man_right.Visible := b;
    end;

  aIm_heli_parts[1, 2].man_right.Visible := b;
  aIm_heli_parts[3, 2].man_right.Visible := b;
  aIm_heli_parts[4, 2].man_right.Visible := b;
  aIm_heli_parts[3, 3].man_left.Visible  := b;
end;





procedure TForm1.Vid_Show_All_Man(b: boolean);
{ displays/hides all the men standing on the buildings                         }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_man[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Man_Safe(b: boolean);
{ displays/hides all the men safely landed on the rightmost building           }
begin
  aIm_man_safe[1].Visible := b;
  aIm_man_safe[2].Visible := b;
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays misses panel in various configurations                              }
  var
    i: integer;
begin
  for i := 1 to 2 do
    aIm_panel_miss[i].Visible := (i <= p);                                      // displays items 1 to p, hides the remaining ones
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    t: integer;
begin
  t := p;

  while t >= 10000 do                                                           // displays "10000" as "0"
    Dec(t, 10000);

  aIm_panel_digit_pos[1].Picture := aIm_panel_number[t div 1000 mod 10].Picture; // thousands
  aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div  100 mod 10].Picture; // hundreds
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div   10 mod 10].Picture; // tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[t          mod 10].Picture; // units
end;





procedure TForm1.Vid_Show_Building(var p: integer; q: integer);
{ displays/hides buildings, fire, smoke depending on the parameters:           }
{ p: building (1-4); q: status (hidden, nornal, danger, broken, visible        }
begin
  case q of
    0: begin                                                                    // totally hidden
         aIm_buildings[p, 1].Visible := false;                                    // smoke:  invisible   buildings: invisible
         aIm_buildings[p, 2].Visible := false;                                    // flames: invisible   quakes:    invisible
       end;
    1: begin                                                                    // normal status
         aIm_buildings[p, 1].Visible := true and (p <> 1);                        // smoke:  invisible   buildings: visible
         aIm_buildings[p, 2].Visible := false;                                    // flames: invisible   quakes:    invisible
       end;
    2: begin                                                                    // danger status
         aIm_buildings[p, 1].Visible := true;                                     // smoke:  visible     buildings: visible
         aIm_buildings[p, 2].Visible := true and (p <> 1);                        // flames: invisible   quakes:    visible
       end;
    3: begin                                                                    // miss status
         aIm_buildings[p, 1].Visible := true and (p = 1);                         // smoke:  visible     buildings: invisible
         aIm_buildings[p, 2].Visible := true and (p = 1);                         // flames: visible     quakes:    visible
       end;
    4: begin                                                                    // totally visible
         aIm_buildings[p, 1].Visible := true;                                     // smoke:  visible     buildings: visible
         aIm_buildings[p, 2].Visible := true;                                     // flames: visible     quakes:    visible
       end;
  end; // case..of
end;





procedure TForm1.Vid_Show_Heli(b: boolean);
{ displays/hides the helicopter                                                }
begin
  if (iPosHeli >= 0) and (iPosHeli <= 6) then
    begin
      aIm_heli[iPosHeli].Visible := b;

      if (iPosHeli >= 1) and (iPosHeli <= 5) then
        begin
          aIm_heli_parts[iPosHeli, 1].rope.Visible      := b and (iMenOnBoard >= 1);
          aIm_heli_parts[iPosHeli, 1].man_left.Visible  := b and (iMenOnBoard >= 1);
          aIm_heli_parts[iPosHeli, 1].man_right.Visible := b and (iMenOnBoard >= 2);
        end;
    end;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time
  if Key = aKeys[4].word then Inp_acl_down    else                              // ACL
  if Key = aKeys[5].word then Inp_left_down   else                              // left
  if Key = aKeys[6].word then Inp_right_down;                                   // right
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if Key = aKeys[4].word then Inp_acl_up    else                                // ACL
  if Key = aKeys[5].word then Inp_left_up   else                                // left
  if Key = aKeys[6].word then Inp_right_up;                                     // right
end;











{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_time_down;
{ Time button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if (not bTimePressed)
    and (not bGameAPressed)
    and (not bGameBPressed)
    and (not bACLPressed) then
    begin
      bTimePressed := true;                                                     // prevents from looping keypresses

      if iMode = 2 then                                                         // Time/1 (hours/minutes) ->
        iMode := 3                                                              //                        -> Time/2 (seconds)
      else                                                                      // whatever mode          ->
        iMode := 2;                                                             //                        -> Time/1 (hours/minutes)

      Gam_Set_Mode;
    end;
end;





procedure TForm1.Inp_time_up;
{ Time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  bTimePressed := false;

  if iMode = 3 then                                                             // Time/2 (seconds) ->
    begin
      iMode := 2; Gam_Set_Mode;                                                 //                  -> Time/1 (hours/minutes)
    end;
end;





procedure TForm1.Inp_game_a_down;
{ Game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if (not bTimePressed)
    and (not bGameAPressed)
    and (not bGameBPressed)
    and (not bACLPressed) then
    begin
      bGameAPressed := true;                                                    // prevents from looping keypresses

      iMode := 4; Gam_Set_Mode;                                                 // whatever mode -> Game A
    end;
end;





procedure TForm1.Inp_game_a_up;
{ Game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;

  bGameAPressed := false;

  if (iMode = 4)
    and (not bTimePressed)
    and (not bGameBPressed)
    and (not bACLPressed) then
    begin
      Vid_Show_Panel_Score(iScore);

      bControls := true;
    end
  else
    bControls := false;
end;





procedure TForm1.Inp_game_b_down;
{ Game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if (not bTimePressed)
    and (not bGameAPressed)
    and (not bGameBPressed)
    and (not bACLPressed) then
    begin
      bGameBPressed := true;                                                    // prevents from looping keypresses

      iMode := 5; Gam_Set_Mode;                                                 // whatever mode -> Game B
    end;
end;





procedure TForm1.Inp_game_b_up;
{ Game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;

  bGameBPressed := false;

  if (iMode = 5)
    and (not bTimePressed)
    and (not bGameAPressed)
    and (not bACLPressed) then
    begin
      Vid_Show_Panel_Score(iScore);

      bControls := true;
    end
  else
    bControls := false;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true;

  if (not bACLPressed) then
    begin
      bACLPressed := true;
                                                                                // prevents from looping keypresses
      iMode := 0; Gam_Set_Mode;
    end;                                                                        // whatever mode -> ACL/1
end;





procedure TForm1.Inp_acl_up;
begin
{ ACL button: RELEASE                                                          }
  btn_acl_down.Visible := false;

  bACLPressed := false;

  if iMode = 0 then                                                             // ACL/1 ->
    begin
      iMode := 1; Gam_Set_Mode;                                                 //       -> ACL/2
    end;
end;





procedure TForm1.Inp_left_down;
{ LEFT button: PRESS                                                           }
begin
  btn_left_down.Visible := true;

  if ( (iMode = 4) or (iMode = 5) )
    and bControls
    and (not bKeyPressed) then
    begin
      bKeyPressed := true;

      if iPosHeli > 0 then
        begin
          Vid_Show_Heli(false);
          Dec(iPosHeli);
          Vid_Show_Heli(true);

          Gam_Check_Collisions;
        end;
    end;

  if ( (iMode = 4) or (iMode = 5) )
    and (iPosHeli = 0) then
    Gam_Game_Over;
end;





procedure TForm1.Inp_left_up;
{ LEFT button: RELEASE                                                         }
begin
  btn_left_down.Visible  := false;

  bKeypressed := false;
end;





procedure TForm1.Inp_right_down;
{ RIGHT button: PRESS                                                          }
begin
  btn_right_down.Visible := true;

  if ( (iMode = 4) or (iMode = 5) )
    and bControls
    and (not bKeyPressed) then
    begin
      bKeyPressed := true;

      if (iPosHeli < 6) then
        begin

          Vid_Show_Heli(false);
          Inc(iPosHeli);
          Vid_Show_Heli(true);

          Gam_Check_Collisions;
        end;
    end;

  if ( (iMode = 4) or (iMode = 5) )
    and (iPosHeli = 6) then
    Gam_Game_Over;
end;





procedure TForm1.Inp_right_up;
{ RIGHT button: RELEASE                                                        }
begin
  btn_right_down.Visible := false;

  bKeypressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all currently enabled timers                                           }
begin
  Timer_ACL.Enabled             := false;
  Timer_Clock_1.Enabled         := false;
  Timer_Clock_2.Enabled         := false;
  Timer_Game.Enabled            := false;
  Timer_Game_Start.Enabled      := false;
  Timer_Miss.Enabled            := false;
  Timer_Rescue_Dropdown.Enabled := false;
  Timer_Rescue_Pickup.Enabled   := false;
end;





procedure TForm1.Gam_Check_Collisions;
{ checks if there's an opportunity to pick a man up from a building            }
begin
  Timer_Rescue_Pickup.Enabled := false;

  case iPosHeli of
    1..4: if aIm_man[iPosHeli].Visible
            and (iMenOnBoard < iMode - 3) then
            begin
              iLoopRescuePickup := 1;
              Timer_Rescue_Pickup.Interval := 1;
              Timer_Rescue_Pickup.Enabled := true;
            end;
    5: if iMenOnBoard > 0 then
         begin
           iLoopRescueDropdown := 1;
           Timer_Rescue_Dropdown.Interval := 1;
           Timer_Rescue_Dropdown.Enabled := true;
         end;
  end; // case..of
end;





procedure TForm1.Gam_Game_Over;
{ game is over due to helicopter crash: action stops and mode changes          }
begin
  Gam_Stop_All_Timers;
  Snd_Silence;
  Snd_Play(pfs_miss);

  bControls := false;

  iMode := 6; Gam_Set_Mode;
end;





procedure TForm1.Gam_Game_Start;
{ sets the stage ready for the game                                            }
  var
    i: integer;
begin
  Vid_Show_All(false);

  for i := 1 to 4 do
    Vid_Show_Building(i, 1);

  iMenOnBoard := 0;
  iPosHeli := 5;
  Vid_Show_Heli(true);

  iScore := 0;
  iMiss := 0;
  Gam_Set_Difficulty;

  for i := 1 to 4 do
    iLoopGameMen[i] := 0;

  // debug
  la_debug.caption := '1=' + inttostr(iLoopGameMen[1]) +
                 '     2=' + inttostr(iLoopGameMen[2]) +
                 '     3=' + inttostr(iLoopGameMen[3]) +
                 '     4=' + inttostr(iLoopGameMen[4]);
                 
  iLoopGameStart := 1;
  Timer_Game_Start.Interval := 1;
  Timer_Game_Start.Enabled := true;
end;





procedure TForm1.Gam_Set_Difficulty;
{ sets the interval between two subsequent men generation                      }
  var
    i,
    iBase: integer;
begin
  // sets the base level (turns 40 with the first "while" statement)
  iBase := 50;

  // decreases by 10 at every 150 points, eventually resulting in negative numbers
  i := iScore;

  while i >= 0 do
    begin
      Dec(iBase, 10);
      Dec(i, 150)
    end;

  // increases by 38 at every 600 points, resulting in an overall decrease of 2 at every 600 points
  i := iScore;

  while i >= 0 do
    begin
      if i >= 600 then Inc(iBase, 38);
      Dec(i, 600)
    end;

  // adjusts the value
  Inc(iBase, Random(3));             // adds random time between 0 and 0,2 seconds
  if iMiss = 1  then Inc(iBase, 10); // increases a little bit if one miss has been made
  if iMode = 5  then Dec(iBase, 5);  // decreases a little bit on Game B
  if iBase > 40 then iBase := 40;
  if iBase < 10 then iBase := 10;

  // declaration
  iLoopGameGenerate := iBase;
end;





procedure TForm1.Gam_Set_Mode;
{ sets handheld in time/game modes, depending on the value of the iMode        }
{ variable                                                                     }
  var
    i: integer;
begin
  case iMode of
    0: begin                                                                    // ACL/1 (freeze)
         Gam_Stop_All_Timers;
         Snd_Silence;

         Vid_Show_All(false);

         bControls := false;
       end;
    1: begin                                                                    // ACL/2 (show all)
         Vid_Show_All(true);

         Timer_ACL.Interval := 2000;
         Timer_ACL.Enabled := true;
       end;
    2: begin                                                                    // Clock/1 (hours/minutes)
         Gam_Stop_All_Timers;
         Snd_Silence;

         Vid_Show_All(false);

         bControls := false;

         Timer_Clock_1.Interval := 1;
         Timer_Clock_1.Enabled := true;
       end;
    3: begin                                                                    // Clock/2 (seconds)
         Gam_Stop_All_Timers;

         im_panel_time_colon.Visible := false;
         aIm_panel_digit_pos[1].picture := nil;
         aIm_panel_digit_pos[2].picture := nil;

         Timer_Clock_2.Interval := 1;
         Timer_Clock_2.Enabled := true;
       end;
    4: begin                                                                    // Game A
         Gam_Stop_All_Timers;
         Snd_Silence;

         bControls := false;

         Gam_Game_Start;
         Vid_Show_Panel_Score(iHiScoreA);
       end;
    5: begin                                                                    // Game B
         Gam_Stop_All_Timers;
         Snd_Silence;

         bControls := false;

         Gam_Game_Start;
         Vid_Show_Panel_Score(iHiScoreB);
       end;
    6: begin                                                                    // Game over
         // does nothing
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_ACLTimer(Sender: TObject);
{ sets the handheld to Time/1 mode                                             }
begin
  iMode := 2; Gam_Set_Mode;
end;





procedure TForm1.Timer_Clock_1Timer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_pm.Visible := true;
    end
  else
    im_panel_time_pm.Visible := false;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  im_panel_time_colon.Visible := not im_panel_time_colon.Visible;               // blinking colon

  Timer_Clock_1.Interval := 500;
end;





procedure TForm1.Timer_Clock_2Timer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  aIm_panel_digit_pos[3].picture := aIm_panel_number[sec div 10].picture;       // seconds: tens
  aIm_panel_digit_pos[4].picture := aIm_panel_number[sec mod 10].picture;       // seconds: units

  Timer_Clock_2.Interval := 1000;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
  var
    i,
    iRnd,
    iOnScreen,
    iMiss:     integer;
    pfsTemp:   PFSoundSample;
begin
  // local variables init
  iMiss := 0;
  iOnScreen := 0;
  pfsTemp := nil;

  // loops action and changes building status to danger and collapse
  for i := 1 to 4 do
    begin
      if iLoopGameMen[i] > 0 then
        Inc(iLoopGameMen[i]);

      if iLoopGameMen[i] = 25 + (Ord(iMode = 4) * 5) then                       // 30 on Game A; 25 on Game B
        Vid_Show_Building(i, 2)
      else
        if iLoopGameMen[i] = 45 + (Ord(iMode = 4) * 5) then                     // 50 on Game A; 45 on Game B
          iMiss := i;

      // debug
      la_debug.caption := '1=' + inttostr(iLoopGameMen[1]) +
                     '     2=' + inttostr(iLoopGameMen[2]) +
                     '     3=' + inttostr(iLoopGameMen[3]) +
                     '     4=' + inttostr(iLoopGameMen[4]);
    end;

  // miss check
  if iMiss <> 0 then
    begin
      Gam_Stop_All_Timers;

      Vid_Show_Building(iMiss, 3);
      aIm_man[iMiss].Visible := false;

      bControls := false;

      iLoopMiss := 1;
      Timer_Miss.Interval := 1;
      Timer_Miss.Enabled := true;
    end;

  // men generation
  if (iMiss = 0) then
    begin
      // counts men on screen
      for i := 1 to 4 do
        if aIm_man[i].Visible then
          Inc(iOnScreen);

      // generation
      Inc(iLoopGame);

      if (iOnScreen <= 2)                                                       // maximum 3 simultaneous men on screen
        and (iLoopGame > iLoopGameGenerate) then
          begin
            repeat iRnd := Random(4) + 1 until
              not aIm_man[iRnd].Visible;

            aIm_man[iRnd].Visible := true;
            iLoopGameMen[iRnd] := 1;

            Gam_Check_Collisions;

            pfsTemp := pfs_danger;

            // restets ticks counter to generating the next man
            iLoopGame := 0;
          end;
    end;

  // sound check
  if (iMiss = 0)
    and (pfsTemp <> nil) then
    Snd_Play(pfsTemp);
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ game action starts, regardless of the fact that the helicopter is or isn't   }
{ in control                                                                   }
begin
  case iLoopGameStart of
    1: begin
         Snd_Play(pfs_start);

         Timer_Game_Start.Interval := 2250;
       end;
    2: begin
         iLoopGame := 21;                                                       // game start is quicker (21), whereas reprise is slower (1)

         Timer_Game.Interval := 100;
         Timer_Game.Enabled := true;

         Timer_Game_Start.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopGameStart);
end;





procedure TForm1.Timer_MissTimer(Sender: TObject);
{ a miss has occurred: plays "miss" sound, decreases lives counter, etc.       }
  var
    i: integer;
begin
  case iLoopMiss of
    1: begin
         Snd_Play(pfs_miss);

         Timer_Miss.Interval := 2500;
       end;
    2: begin
         Inc(iMiss);

         Vid_Show_Panel_Miss(iMiss);

         if iMiss < 2 then                                                      // misses counter check
           begin                                                                  // game reprise
             Vid_Show_All_Heli(false);
             Vid_Show_All_Man(false);
             Vid_Show_All_Man_Safe(false);

             for i := 1 to 4 do
               begin
                 Vid_Show_Building(i, 1);
                 iLoopGameMen[i] := 0;
               end;

             iMenOnBoard := 0;
             iPosHeli := 5;
             Vid_Show_Heli(true);

             bControls := true;

             Gam_Set_Difficulty;

             iLoopGame := 1;
             Timer_Game.Interval := 100;
             Timer_Game.Enabled := true;
           end
         else                                                                   // else
           begin                                                                  // game over
             iMode := 6; Gam_Set_Mode;
           end;

         Timer_Miss.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopMiss);
end;





procedure TForm1.Timer_Rescue_DropdownTimer(Sender: TObject);
{ one or two men are dropped to the heliport                                   }
  var
    i: integer;
begin
  case iLoopRescueDropdown of
    1: begin
         bControls := false;

         Timer_Rescue_Dropdown.Interval := 150;
       end;
    2: begin
         // score and hiscore management (hiscores are saved in-game)
         Inc(iScore, iMenOnBoard * 10);

         if (iMode = 4) and (iScore > iHiScoreA) then begin iHiScoreA := iScore; iReg.WriteInteger('', 'HiScore1', iHiScoreA); end;
         if (iMode = 5) and (iScore > iHiScoreB) then begin iHiScoreB := iScore; iReg.WriteInteger('', 'HiScore2', iHiScoreB); end;

         Vid_Show_Panel_Score(iScore);

         // helicopter and men management
         aIm_man_safe[1].Visible := true;
         aIm_man_safe[2].Visible := (iMenOnBoard = 2);

         iMenOnBoard := 0;
         Vid_Show_Heli(true);

         // game over check
         if iScore >= 9990 then
           Gam_Game_Over                                                        // Gam_Stop_All_Timers and Snd_Play(pfs_miss) are included here
         else
           begin
             Gam_Set_Difficulty;

             Timer_Rescue_Dropdown.Interval := 250;
           end;
       end;
    3: begin
         Vid_Show_All_Man_Safe(false);

         bControls := true;

         Timer_Game.Enabled := true;

         Timer_Rescue_Dropdown.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopRescueDropdown);
end;





procedure TForm1.Timer_Rescue_PickupTimer(Sender: TObject);
{ one man is rescued from a building                                           }
begin
  case iLoopRescuePickup of
    1: Timer_Rescue_Pickup.Interval := 240;
    2: begin
         // sets all variables for the current procedure
         aIm_heli_parts[iPosHeli, 1].rope.Visible := true;

         Snd_Play(pfs_bar);

         iLoopGameMen[iPosHeli] := 0;                                           // building goes back to normality
         Inc(iMenOnBoard);                                                      // adds one man on board
         Inc(iLoopGame);                                                        // main game action proceeds by one tick

         bControls := false;

         Timer_Game.Enabled := false;
       end;
    3: begin
         case aBuildings_height[iPosHeli] of
           1: begin
                aIm_man[iPosHeli].Visible := false;
                Vid_Show_Building(iPosHeli, 1);
                Vid_Show_Heli(true);

                // timer about to stop...
                iLoopRescuePickup := 100;
              end;
           2, 3: aIm_heli_parts[iPosHeli, 2].rope.Visible := true;
         end; // case..of

         Snd_Play(pfs_bar);

         Inc(iLoopGame);
       end;
    4: begin
         case aBuildings_height[iPosHeli] of
           2: begin
                aIm_man[iPosHeli].Visible := false;
                aIm_heli_parts[iPosHeli, 2].man_right.Visible := true;
                Vid_Show_Building(iPosHeli, 1);
              end;
           3: aIm_heli_parts[iPosHeli, 3].rope.Visible := true;
         end; // case..of

         Snd_Play(pfs_bar);

         Inc(iLoopGame);
       end;
    5: begin
         case aBuildings_height[iPosHeli] of
           2: begin
                aIm_heli_parts[iPosHeli, 2].rope.Visible := false;
                aIm_heli_parts[iPosHeli, 2].man_right.Visible := false;
                Vid_Show_Building(iPosHeli, 1);
                Vid_Show_Heli(true);

                // timer about to stop...
                iLoopRescuePickup := 100;
              end;
           3: begin
                aIm_man[iPosHeli].Visible := false;
                aIm_heli_parts[iPosHeli, 3].man_left.Visible := true;
                Vid_Show_Building(iPosHeli, 1);
              end;
         end; // case..of

         Snd_Play(pfs_bar);

         Inc(iLoopGame);
       end;
    6: begin
         aIm_heli_parts[iPosHeli, 3].rope.Visible := false;
         aIm_heli_parts[iPosHeli, 3].man_left.Visible := false;
         aIm_heli_parts[iPosHeli, 2].man_right.Visible := true;

         Snd_Play(pfs_bar);

         Inc(iLoopGame);
       end;
    7: begin
         aIm_heli_parts[iPosHeli, 2].rope.Visible := false;
         aIm_heli_parts[iPosHeli, 2].man_right.Visible := false;
         Vid_Show_Building(iPosHeli, 1);
         Vid_Show_Heli(true);

         // timer about to stop...
         iLoopRescuePickup := 100;

         Snd_Play(pfs_bar);

         Inc(iLoopGame);
       end;
  101: begin
         // action reprises
         bControls := true;

         Timer_Game.Enabled := true;

         Timer_Rescue_Pickup.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopRescuePickup);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Height := 16;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
